#include "memorydevice.h"
#include "misc.h"
#include <vector>


std::string MemoryDevice::readString(HANDLE process, long address, int &err,
    unsigned int len)
{
    std::string fullstr;
    unsigned char buffer = 0;
    SIZE_T bytesread;
    err = 0;
    int success;

    long stroffset = 0;
    unsigned int count = 0;
    while( 1 ) // read until we hit a NULL
    {
        success = ReadProcessMemory(process, (LPCVOID)(address + stroffset),
            (void*)&buffer, sizeof(unsigned char), &bytesread);
        stroffset++;

        fullstr.push_back(buffer);
        count++;

        if( !len && buffer == '\0' )
            break;

        if( len && count >= len )
        {
            fullstr.push_back('\0');
            break;
        }
    }

    if( success == 0 )
        err = MEMORY_READ_FAIL;

    return fullstr;
}

std::wstring MemoryDevice::readUString(HANDLE process, long address, int &err,
    unsigned int len)
{
    std::wstring fullstr;
    wchar_t buffer = 0;
    SIZE_T bytesread;
    err = 0;
    int success;


    long stroffset = 0;
    unsigned int count = 0;
    while( 1 ) // read until we hit a NULL
    {
        success = ReadProcessMemory(process, (LPCVOID)(address + stroffset),
            (void*)&buffer, sizeof(wchar_t), &bytesread);
        stroffset += sizeof(wchar_t);

        fullstr.push_back(buffer);
        count++;

        if( buffer == 0 )
            break;

        if( len != 0 && count >= len )
        {
            fullstr.push_back('\0');
            break;
        }
    }

    if( success == 0 )
        err = MEMORY_READ_FAIL;

    return fullstr;
}


std::string MemoryDevice::readStringPtr(HANDLE process, long address,
    long offset, int &err, unsigned int len)
{
    std::string fullstr;
    char buffer = 0;
    SIZE_T bytesread;
    err = 0;
    int success;
    long real_addr = 0;

    // read the value of the pointer, first
    success = ReadProcessMemory(process, (LPCVOID)address,
        (void*)&real_addr, sizeof(long), &bytesread);

    if( success == 0 ) {
        err = MEMORY_READ_FAIL;
        return 0; }

    long stroffset = 0;
    unsigned int count = 0;
    while( 1 )
    {
        success = ReadProcessMemory(process,
            (LPCVOID)(real_addr + offset + stroffset), (void*)&buffer,
            sizeof(char), &bytesread);
        stroffset += sizeof(char);

        fullstr.push_back(buffer);
        count++;

        if( !len && buffer == '\0' )
            break;

        if( len && count >= len )
        {
            fullstr.push_back('\0');
            break;
        }
    }

    if( success == 0 )
        err = MEMORY_READ_FAIL;

    return fullstr;
}

std::wstring MemoryDevice::readUStringPtr(HANDLE process, long address,
    long offset, int &err, unsigned int len)
{
    std::wstring fullstr;
    wchar_t buffer = 0;
    SIZE_T bytesread;
    err = 0;
    int success;
    long real_addr = 0;

    // read the value of the pointer, first
    success = ReadProcessMemory(process, (LPCVOID)address,
        (void*)&real_addr, sizeof(long), &bytesread);

    if( success == 0 ) {
        err = MEMORY_READ_FAIL;
        return 0; }

    long stroffset = 0;
    unsigned int count = 0;
    while( 1 )
    {
        success = ReadProcessMemory(process,
            (LPCVOID)(real_addr + offset + stroffset), (void*)&buffer,
            sizeof(wchar_t), &bytesread);
        stroffset += sizeof(wchar_t);

        fullstr.push_back(buffer);
        count++;

        if( !len && buffer ==0 )
            break;

        if( len && count >= len )
        {
            fullstr.push_back('\0');
            break;
        }
    }

    if( success == 0 )
        err = MEMORY_READ_FAIL;

    return fullstr;
}

void MemoryDevice::writeString(HANDLE process, long address, char *data, int &err, unsigned int len)
{
	SIZE_T byteswritten = 0;
	err = 0;
	int success = 0;
	DWORD old;

	VirtualProtectEx(process, (void *)address, (size_t)len, PAGE_READWRITE, &old);
	success = WriteProcessMemory(process, (void *)address,
	(void*)data, (size_t)len, &byteswritten);
	VirtualProtectEx(process, (void *)address, (size_t)len, old, &old);

	if( success == 0 )
		err = MEMORY_WRITE_FAIL;
}

void MemoryDevice::writeStringPtr(HANDLE process, long address, long offset, char *data,
	int &err, unsigned int len)
{
	long real_addr = 0;
	SIZE_T bytesread = 0;
	SIZE_T byteswritten = 0;
	int success = 0;
	err = 0;
	DWORD old;

	// find the real (base) address to write to
	success = ReadProcessMemory(process, (LPCVOID)address,
	(void*)&real_addr, sizeof(long), &bytesread);

	if( success == 0 ) {
		err = MEMORY_READ_FAIL;
		return; }

	// now write to that address + offset
	VirtualProtectEx(process, (void *)(real_addr + offset), (size_t)len, PAGE_READWRITE, &old);
	success = WriteProcessMemory(process,
		(void *)(real_addr + offset), (void*)data,
		(size_t)len, &byteswritten);
	VirtualProtectEx(process, (void *)(real_addr + offset), (size_t)len, old, &old);

	if( success == 0 )
		err = MEMORY_WRITE_FAIL;
}
